@@ -1,4 +1,4 @@
-from .bees_algorithm import Bee, BeesAlgorithm, visualize_steps
+from .bees_algorithm import Bee, BeesAlgorithm
 from .bees_algorithm_parallel_testing import BeesAlgorithmTester
 from .bees_algorithm_parallel_algorithm import ParallelBeesAlgorithm, FullyParallelBeesAlgorithm
 
@@ -7,6 +7,5 @@ __all__=[
 	'BeesAlgorithm',
 	'BeesAlgorithmTester',
 	'ParallelBeesAlgorithm', 
-	'FullyParallelBeesAlgorithm',
-	'visualize_steps'
+	'FullyParallelBeesAlgorithm'
 ]
