@@ -1,12 +1,13 @@
 #!/usr/bin/env python3
 
-'''
-Code developed by Luca Baronti (lbaronti[at]gmail.com)
-'''
-
 import bees_algorithm as ba
 import math, time, copy, multiprocessing, sys
 
+__author__      = 'Luca Baronti'
+__maintainer__  = 'Luca Baronti'
+__license__     = 'GPLv3'
+__version__     = '1.0.1'
+
 def threadedGenerateScout(score_function,range_min,range_max,stlim,initial_ngh):
 	bee=ba.Bee(range_min,range_max,stlim,initial_ngh,isForager=False,centre=None)
 	bee.score=score_function(bee.values)
