@@ -1,12 +1,13 @@
 #!/usr/bin/env python3
 
-'''
-Code developed by Luca Baronti (lbaronti[at]gmail.com)
-'''
-
 import bees_algorithm
 import time, multiprocessing
 
+__author__      = 'Luca Baronti'
+__maintainer__  = 'Luca Baronti'
+__license__     = 'GPLv3'
+__version__     = '1.0.1'
+
 def threaded_run(ba_instance,max_iteration,max_score,verbose):
 	iteration, _ =ba_instance.performFullOptimisation(max_iteration=max_iteration,max_score=max_score,verbose=0)
 	result=ba_instance.best_solution
